/* Copyright (C) 2013-2016 Vincent Forest (vaplv@free.fr)
 *
 * The RSys library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The RSys library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with the RSys library. If not, see <http://www.gnu.org/licenses/>. */

#include "float44.h"

float
f44_inverse(float* dst, const float* m)
{
  float tmp[9];
  float det_012[4], det_023[4], det_123[4], det_013[4];
  float cofacts[4];
  float det, rcp_det, mpmp_rcp_det[4], pmpm_rcp_det[4];
  ASSERT( dst && m );

  /* Define the 3x3 sub matrices and compute their determinants */
  #define C3( Dst, Mat, ICol, X, Y, Z ) \
    (Dst)[0] = Mat[ICol*4+X], (Dst)[1] = Mat[ICol*4+Y], (Dst)[2] = Mat[ICol*4+Z]
  C3(tmp+0 ,m, 1, 0, 1, 2); C3(tmp+3, m, 2, 0, 1, 2); C3(tmp+6, m, 3, 0, 1, 2);
  det_012[0] = f33_det(tmp);
  C3(tmp+0 ,m, 0, 0, 1, 2); C3(tmp+3, m, 2, 0, 1, 2); C3(tmp+6, m, 3, 0, 1, 2);
  det_012[1] = f33_det(tmp);
  C3(tmp+0 ,m, 0, 0, 1, 2); C3(tmp+3, m, 1, 0, 1, 2); C3(tmp+6, m, 3, 0, 1, 2);
  det_012[2] = f33_det(tmp);
  C3(tmp+0 ,m, 0, 0, 1, 2); C3(tmp+3, m, 1, 0, 1, 2); C3(tmp+6, m, 2, 0, 1, 2);
  det_012[3] = f33_det(tmp);

  C3(tmp+0 ,m, 1, 0, 2, 3); C3(tmp+3, m, 2, 0, 2, 3); C3(tmp+6, m, 3, 0, 2, 3);
  det_023[0] = f33_det(tmp);
  C3(tmp+0 ,m, 0, 0, 2, 3); C3(tmp+3, m, 2, 0, 2, 3); C3(tmp+6, m, 3, 0, 2, 3);
  det_023[1] = f33_det(tmp);
  C3(tmp+0 ,m, 0, 0, 2, 3); C3(tmp+3, m, 1, 0, 2, 3); C3(tmp+6, m, 3, 0, 2, 3);
  det_023[2] = f33_det(tmp);
  C3(tmp+0 ,m, 0, 0, 2, 3); C3(tmp+3, m, 1, 0, 2, 3); C3(tmp+6, m, 2, 0, 2, 3);
  det_023[3] = f33_det(tmp);

  C3(tmp+0 ,m, 1, 1, 2, 3); C3(tmp+3, m, 2, 1, 2, 3); C3(tmp+6, m, 3, 1, 2, 3);
  det_123[0] = f33_det(tmp);
  C3(tmp+0 ,m, 0, 1, 2, 3); C3(tmp+3, m, 2, 1, 2, 3); C3(tmp+6, m, 3, 1, 2, 3);
  det_123[1] = f33_det(tmp);
  C3(tmp+0 ,m, 0, 1, 2, 3); C3(tmp+3, m, 1, 1, 2, 3); C3(tmp+6, m, 3, 1, 2, 3);
  det_123[2] = f33_det(tmp);
  C3(tmp+0 ,m, 0, 1, 2, 3); C3(tmp+3, m, 1, 1, 2, 3); C3(tmp+6, m, 2, 1, 2, 3);
  det_123[3] = f33_det(tmp);

  C3(tmp+0 ,m, 1, 0, 1, 3); C3(tmp+3, m, 2, 0, 1, 3); C3(tmp+6, m, 3, 0, 1, 3);
  det_013[0] = f33_det(tmp);
  C3(tmp+0 ,m, 0, 0, 1, 3); C3(tmp+3, m, 2, 0, 1, 3); C3(tmp+6, m, 3, 0, 1, 3);
  det_013[1] = f33_det(tmp);
  C3(tmp+0 ,m, 0, 0, 1, 3); C3(tmp+3, m, 1, 0, 1, 3); C3(tmp+6, m, 3, 0, 1, 3);
  det_013[2] = f33_det(tmp);
  C3(tmp+0 ,m, 0, 0, 1, 3); C3(tmp+3, m, 1, 0, 1, 3); C3(tmp+6, m, 2, 0, 1, 3);
  det_013[3] = f33_det(tmp);
  #undef C3

  f4_set(cofacts, f4(tmp, -det_012[0], det_012[1], -det_012[2], det_012[3]));

  /* Compute the determinant of src */
  det = f4_dot(cofacts, f4(tmp, m[3], m[7], m[11], m[15]));

  /* Invert the matrix */
  if( det == 0.f )
    return det;

  rcp_det = 1.f / det;
  f4_set(mpmp_rcp_det, f4(tmp,-rcp_det, rcp_det,-rcp_det, rcp_det));
  f4_set(pmpm_rcp_det, f4(tmp, rcp_det,-rcp_det, rcp_det,-rcp_det));
  f4_mul(f44_col_ptr(dst, 0), det_123, pmpm_rcp_det);
  f4_mul(f44_col_ptr(dst, 1), det_023, mpmp_rcp_det);
  f4_mul(f44_col_ptr(dst, 2), det_013, pmpm_rcp_det);
  f4_mul(f44_col_ptr(dst, 3), det_012, mpmp_rcp_det);
  return det;
}
