/* Copyright (C) 2013-2016 Vincent Forest (vaplv@free.fr)
 *
 * The RSys library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The RSys library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with the RSys library. If not, see <http://www.gnu.org/licenses/>. */

#include "float3.h"
#include "rsys.h"
#include <float.h>

#define CHECK_F3(a, b)                                                         \
  {                                                                            \
    float* a__ = (a);                                                          \
    float* b__ = (b);                                                          \
    CHECK((a__)[0], (b__)[0]);                                                 \
    CHECK((a__)[1], (b__)[1]);                                                 \
    CHECK((a__)[2], (b__)[2]);                                                 \
  } (void)0

int
main(int argc, char** argv)
{
  float a[3], b[3], dst[3], f, c[3];
  (void)argc, (void)argv;

  f3_set(a, f3_splat(c, -1.f));
  CHECK(a[0], -1.f);
  CHECK(a[1], -1.f);
  CHECK(a[2], -1.f);
  f3_set(a, f3(c, 0.f, 1.f, 2.f));
  CHECK(a[0], 0.f);
  CHECK(a[1], 1.f);
  CHECK(a[2], 2.f);
  f3_splat(a, -2.f);
  CHECK(a[0], -2.f);
  CHECK(a[1], -2.f);
  CHECK(a[2], -2.f);

  f3_set(a, f3(c, -1.f, 2.f, -3.f));
  CHECK_F3(f3_minus(b, a), f3(c, 1.f, -2.f, 3.f));
  CHECK_F3(b, f3(c, 1.f, -2.f, 3.f));

  CHECK_F3(f3_add(dst, a, b), f3_splat(c, 0.f));
  CHECK_F3(dst, f3_splat(c, 0.f));
  CHECK_F3(f3_addf(dst, a, 1), f3(c, 0.f, 3.f, -2.f));
  CHECK_F3(dst, f3(c, 0.f, 3.f, -2.f));
  CHECK_F3(f3_sub(dst, a, b), f3(c, -2.f, 4.f, -6.f));
  CHECK_F3(dst, f3(c, -2.f, 4.f, -6.f));
  CHECK_F3(f3_subf(dst, a, 1), f3(c, -2.f, 1.f, -4.f));
  CHECK_F3(dst, f3(c, -2.f, 1.f, -4.f));
  CHECK_F3(f3_mulf(dst, a, 2), f3(c, -2.f, 4.f, -6.f));
  CHECK_F3(dst, f3(c, -2.f, 4.f, -6.f));
  CHECK_F3(f3_mul(dst, a, b), f3(c, -1.f, -4.f, -9.f));
  CHECK_F3(dst, f3(c, -1.f, -4.f, -9.f));
  CHECK_F3(f3_div(dst, dst, a), f3(c, 1.f, -2.f, 3.f));
  CHECK_F3(dst, f3(c, 1.f, -2.f, 3.f));
  CHECK_F3(f3_divf(dst, a, 2.f), f3(c, -0.5f, 1.f, -1.5f));
  CHECK_F3(dst, f3(c, -0.5f, 1.f, -1.5f));

  f3_set(a, f3(c, 0.f, 1.f, 2.f));
  f3_set(b, f3(c, 1.f, 2.f, -1.f));
  CHECK_F3(f3_lerp(dst, a, b, 0.5f), f3(c, 0.5f, 1.5f, 0.5f));
  CHECK_F3(dst, f3(c, 0.5f, 1.5f, 0.5f));
  CHECK(f3_sum(b), 2.f);
  CHECK(f3_dot(a, b), 0.f);
  CHECK(eq_epsf(f3_len(a), (float)sqrt(5.0), FLT_EPSILON), 1);

  CHECK(f3_is_normalized(a), 0);
  f = f3_normalize(dst, a);
  CHECK(f3_is_normalized(a), 0);
  NCHECK(f3_is_normalized(dst), 0);
  CHECK(eq_epsf(f, (float)sqrt(5.0), FLT_EPSILON), 1);
  CHECK(eq_epsf(dst[0], a[0] / f, FLT_EPSILON), 1);
  CHECK(eq_epsf(dst[1], a[1] / f, FLT_EPSILON), 1);
  CHECK(eq_epsf(dst[2], a[2] / f, FLT_EPSILON), 1);

  CHECK(f3_eq(a, a), 1);
  CHECK(f3_eq(a, b), 0);
  CHECK(f3_eq(a, f3(c, b[0], a[1], a[2])), 0);
  CHECK(f3_eq(a, f3(c, a[0], b[1], a[2])), 0);
  CHECK(f3_eq(a, f3(c, a[0], a[1], b[2])), 0);

  f3_set(b, a);
  f3_add(b, b, f3_splat(c, FLT_EPSILON * 0.5f));
  CHECK(f3_eq(a, b), 0);
  CHECK(f3_eq_eps(a, b, FLT_EPSILON), 1);
  CHECK(f3_eq_eps(a, b, FLT_EPSILON * 0.25f), 0);
  f3_set(a, f3(c, 1.f, -2.f, 2.f));
  f3_set(b, f3(c, 3.f, 1.f, -1.5f));
  CHECK_F3(f3_cross(dst, a, b), f3(c, 1.f, 7.5f, 7.f));
  CHECK_F3(dst, f3(c, 1.f, 7.5f, 7.f));

  f3(a, 0.f, 1.f, 0.f);
  f3(b, 0.f, 0.f, 1.f);
  CHECK_F3(f3_cross(a, a, b), f3(c, 1.f, 0.f, 0.f)); 

  f3_set(a, f3(c, 1.f, -2.f, 1.5f));
  f3_set(b, f3(c, -3.f, 1.f, -1.5f));
  CHECK_F3(f3_max(dst, a, b), f3(c, 1.f, 1.f, 1.5f));
  CHECK_F3(dst, f3(c, 1.f, 1.f, 1.5f));
  CHECK_F3(f3_min(dst, a, b), f3(c, -3.f, -2.f, -1.5f));
  CHECK_F3(dst, f3(c, -3.f, -2.f, -1.5f));
  return 0;
}

