/* Copyright (C) 2013-2016 Vincent Forest (vaplv@free.fr)
 *
 * The RSys library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The RSys library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with the RSys library. If not, see <http://www.gnu.org/licenses/>. */

#include "str.h"
#include "test_utils.h"
#include <string.h>

int
main(int argc, char** argv)
{
  struct mem_allocator allocator_proxy;
  struct str str, str2;
  (void)argc, (void)argv;

  mem_init_proxy_allocator(&allocator_proxy, &mem_default_allocator);

  str_init(&allocator_proxy, &str);
  CHECK(strcmp(str_get(&str), ""), 0);
  CHECK(str_len(&str), 0);
  CHECK(str_is_empty(&str), 1);

  CHECK(str_set(&str, "Foo"), RES_OK);
  CHECK(strcmp(str_get(&str), "Foo"), 0);
  CHECK(str_len(&str), strlen("Foo"));
  CHECK(str_is_empty(&str), 0);

  str_clear(&str);
  CHECK(strcmp(str_get(&str), ""), 0);
  CHECK(str_len(&str), 0);

  CHECK(str_set(&str, __FILE__), RES_OK);
  CHECK(strcmp(str_get(&str), __FILE__), 0);

  str_release(&str);
  str_init(&allocator_proxy, &str);
  CHECK(strcmp(str_get(&str), ""), 0);
  CHECK(str_len(&str), 0);

  CHECK(str_set(&str, "Hello world!"), RES_OK);
  CHECK(strcmp(str_get(&str), "Hello world!"), 0);
  NCHECK(str_insert(&str, 13, " insert"), RES_OK);
  CHECK(str_insert(&str, 12, " insert"), RES_OK);
  CHECK(strcmp(str_get(&str), "Hello world! insert"), 0);

  CHECK(str_insert(&str, 6, "abcdefgh "), RES_OK);
  CHECK(strcmp(str_get(&str), "Hello abcdefgh world! insert"), 0);
  CHECK(str_insert(&str, 0, "ABC "), RES_OK);
  CHECK(strcmp(str_get(&str), "ABC Hello abcdefgh world! insert"), 0);
  CHECK(str_insert(&str, 11, "0123456789ABCDEF"), RES_OK);
  CHECK(strcmp(str_get(&str),
    "ABC Hello a0123456789ABCDEFbcdefgh world! insert"), 0);
  CHECK(str_len(&str),
    strlen("ABC Hello a0123456789ABCDEFbcdefgh world! insert"));

  CHECK(str_set(&str, "Hello world!"), RES_OK);
  CHECK(str_append(&str, " Append"), RES_OK);
  CHECK(strcmp(str_get(&str), "Hello world! Append"), 0);
  CHECK(str_append(&str, "!"), RES_OK);
  CHECK(strcmp(str_get(&str), "Hello world! Append!"), 0);

  NCHECK(str_insert_char(&str, 21, 'a'), RES_OK);
  CHECK(strcmp(str_get(&str), "Hello world! Append!"), 0);
  CHECK(str_insert_char(&str, 20, 'a'), RES_OK);
  CHECK(strcmp(str_get(&str), "Hello world! Append!a"), 0);
  CHECK(str_insert_char(&str, 0, '0'), RES_OK);
  CHECK(strcmp(str_get(&str), "0Hello world! Append!a"), 0);
  CHECK(str_insert_char(&str, 13, 'A'), RES_OK);
  CHECK(strcmp(str_get(&str), "0Hello world!A Append!a"), 0);

  CHECK(str_append_char(&str, 'z'), RES_OK);
  CHECK(strcmp(str_get(&str), "0Hello world!A Append!az"), 0);

  CHECK(str_reserve(&str, 128), RES_OK);
  CHECK(str_reserve(&str, 0), RES_OK);
  CHECK(strcmp(str_get(&str), "0Hello world!A Append!az"), 0);

  CHECK(str_insert_char(&str, 13, '\0'), RES_OK);
  CHECK(strcmp(str_get(&str), "0Hello world!"), 0);
  CHECK(str_len(&str), strlen("0Hello world!"));
  CHECK(str_append_char(&str, '\0'), RES_OK);
  CHECK(strcmp(str_get(&str), "0Hello world!"), 0);
  CHECK(str_len(&str), strlen("0Hello world!"));

  str_init(&allocator_proxy, &str2);
  str_copy(&str2, &str);
  CHECK(strcmp(str_cget(&str2), "0Hello world!"), 0);
  CHECK(strcmp(str_cget(&str), "0Hello world!"), 0);

  CHECK(str_eq(&str, &str2), 1);

  CHECK(str_hash(&str), str_hash(&str2));
  str_clear(&str2);
  NCHECK(str_hash(&str), str_hash(&str2));

  str_copy_and_clear(&str2, &str);
  CHECK(strcmp(str_cget(&str2), "0Hello world!"), 0);
  CHECK(str_len(&str2), strlen(str_cget(&str2)));
  CHECK(str_len(&str), 0);
  str_copy_and_release(&str, &str2);
  CHECK(strcmp(str_cget(&str), "0Hello world!"), 0);

  str_init(&allocator_proxy, &str2);
  str_set(&str2, "ABC Hello a0123456789ABCDEFbcdefgh world! insert");
  CHECK(str_eq(&str, &str2), 0);
  CHECK(str_cmp(&str, &str2), strcmp(str_cget(&str), str_cget(&str2)));

  NCHECK(str_hash(&str), str_hash(&str2));
  str_copy_and_clear(&str, &str2);
  CHECK(0, strcmp
    (str_cget(&str), "ABC Hello a0123456789ABCDEFbcdefgh world! insert"));
  CHECK(str_len(&str), strlen(str_cget(&str)));

  str_set(&str2, "Hello world!");
  CHECK(strcmp(str_cget(&str2), "Hello world!"), 0);
  str_clear(&str2);
  str_copy_and_clear(&str, &str2);
  CHECK(str_len(&str), 0);
  CHECK(str_len(&str2), 0);
  CHECK(strlen(str_cget(&str)), 0);

  str_copy_and_release(&str2, &str2);

  str_set(&str, "Hello World!");
  str_copy_and_clear(&str, &str);
  CHECK(str_len(&str), 0);

  str_release(&str);

  check_memory_allocator(&allocator_proxy);
  mem_shutdown_proxy_allocator(&allocator_proxy);
  CHECK(mem_allocated_size(), 0);
  return 0;
}
