/* Copyright (C) 2015-2016 CNRS
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>. */

#ifndef SCHIFF_ARGS_H
#define SCHIFF_ARGS_H

#include "schiff_geometry.h"
#include <rsys/rsys.h>
#include <star/sschiff.h>

struct schiff_args {
  const char* output_filename; /* File in which output data are stored */
  struct schiff_optical_properties* properties; /* List of properties */
  double* wavelengths; /* List of wavelengths to integrate */

  double characteristic_length; /* Characteristic length of the soft particles */

  /* List of parsed geometry distribution parameters and its associated random
   * variate */
  struct schiff_geometry* geoms;
  struct ssp_ran_discrete* ran_geoms;

  unsigned ngeoms_dump; /* # sampled geometries to dump */
  unsigned nrealisations; /* # realisation */
  unsigned ndirs; /* Number of directions to sample per realisation */
  unsigned nangles; /* Number of scattering angles */
  unsigned nangles_inv; /* Number of angles in the the inverse phase function */

  unsigned nthreads; /* Hint on the number of thread to use */
};

static const struct schiff_args SCHIFF_ARGS_NULL = {
  NULL, /* Output filename */
  NULL, /* List of optical properties */
  NULL, /* List of wavelength to integrate */
  -1.0, /* Caracteristic length */
  NULL, /* List of Schiff geometries */
  NULL, /* Schiff geometry random variates */
  0, /* # Dumped geometries */
  10000, /* # Sampled geometries */
  100, /* # Sampled directions per geometry */
  1000, /* # Scattering angles */
  2000, /* # angles in the inv cumulative phase function */
  SSCHIFF_NTHREADS_DEFAULT
};

extern LOCAL_SYM res_T
schiff_args_init
  (struct schiff_args* args,
   const int argc,
   char** argv);

extern LOCAL_SYM void
schiff_args_release
  (struct schiff_args* args);

#endif /* SCHIFF_ARGS_H */

