/* Copyright (C) |Meso|Star> 2015-2016 (contact@meso-star.com)
 *
 * This software is a computer program whose purpose is to describe a
 * virtual 3D environment that can be ray-traced and sampled both robustly
 * and efficiently.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty and the software's author, the holder of the
 * economic rights, and the successive licensors have only limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading, using, modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean that it is complicated to manipulate, and that also
 * therefore means that it is reserved for developers and experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and, more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms. */

#ifndef S3D_C_H
#define S3D_C_H

#include "s3d.h"
#include "s3d_backend.h"

#include <rsys/rsys.h>

static FINLINE res_T
rtc_error_to_res_T(const enum RTCError err)
{
  switch(err) {
    case RTC_NO_ERROR: return RES_OK;
    case RTC_UNKNOWN_ERROR: return RES_UNKNOWN_ERR;
    case RTC_INVALID_ARGUMENT: return RES_BAD_ARG;
    case RTC_INVALID_OPERATION: return RES_BAD_ARG;
    case RTC_OUT_OF_MEMORY: return RES_MEM_ERR;
    case RTC_UNSUPPORTED_CPU: return RES_BAD_ARG;
    default: FATAL("Unreachable code\n"); break;
  }
}

static INLINE unsigned
s3d_type_get_dimension(const enum s3d_type type)
{
  switch(type) {
    case S3D_FLOAT: return 1;
    case S3D_FLOAT2: return 2;
    case S3D_FLOAT3: return 3;
    case S3D_FLOAT4: return 4;
    default: FATAL("Unreachable code\n"); break;
  }
}

#endif /* S3D_C_H */

