/* Copyright (C) |Meso|Star> 2015-2016 (contact@meso-star.com)
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty and the software's author, the holder of the
 * economic rights, and the successive licensors have only limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading, using, modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean that it is complicated to manipulate, and that also
 * therefore means that it is reserved for developers and experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and, more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms. */

#include "ssp.h"
#include "test_ssp_utils.h"

#define NSAMPS 128

int
main(int argc, char** argv)
{
  struct ssp_rng* rng;
  struct mem_allocator allocator;
  float samps[NSAMPS][4];
  float* f = NULL;
  int i = 0, j = 0;
  (void)argc, (void)argv;

  mem_init_proxy_allocator(&allocator, &mem_default_allocator);

  CHECK(ssp_rng_create(&allocator, &ssp_rng_mt19937_64, &rng), RES_OK);

  FOR_EACH(i, 0, NSAMPS) {
    f = ssp_ran_sphere_uniform(rng, samps[i]);
    CHECK(f, samps[i]);
    CHECK(f3_is_normalized(f), 1);
    CHECK(eq_epsf(samps[i][3], (float)(1.0/(4.0*PI)), 1.e-6f), 1);
    CHECK(eq_epsf(samps[i][3], ssp_ran_sphere_uniform_pdf(), 1.e-6f), 1);
    FOR_EACH(j, 0, i) {
      CHECK(f3_eq_eps(samps[j], samps[i], 1.e-6f), 0);
    }
  }

  ssp_rng_ref_put(rng);
  check_memory_allocator(&allocator);
  mem_shutdown_proxy_allocator(&allocator);

  CHECK(mem_allocated_size(), 0);
  return 0;
}
